@@ -27,16 +27,16 @@ NOTE_TYPES = ("right", "left", "single", "both")
 WALL_TYPES = {
     # name: (index, center_offset)
     # index 0-4 are the same "slideType" used in the JSON in the "slides" list
-    "wall_right": (0, [4, -2]),
-    "wall_left": (1, [-4, -2]),
-    "angle_right": (2, [2, -2]),
+    "wall_right": (0, [4.05, -1.85]),
+    "wall_left": (1, [-3.95, -1.85]),
+    "angle_right": (2, [2.5, -2.2]),
     "center": (3, [0, -1.3]),
-    "angle_left": (4, [-2, -2]),
+    "angle_left": (4, [-2.5, -2.2]),
     # 5-99: placeholder for future slide types
     # Since there can be only be one wall anyway, just treat these three the same
-    "crouch": (100, [0, 7.5]),
+    "crouch": (100, [0.1, 7.5]),
     "square": (101, [0, -2]),
-    "triangle": (102, [0, -4.3]),
+    "triangle": (102, [0, -4.7]),
 }
 WALL_LOOKUP = {id: name for name, (id, _) in WALL_TYPES.items()}
 WALL_OFFSETS = {id: np.array(offset + [0]) for _, (id, offset) in WALL_TYPES.items()}
@@ -116,7 +116,7 @@ class DataContainer:
 def coord_from_synth(bpm: float, startMeasure: float, coord: list[float]) -> "numpy array (3)":
     return np.array([
         (coord[0] - X_OFFSET) / GRID_SCALE,
-        (coord[1]-Y_OFFSET) / GRID_SCALE,
+        (coord[1] - Y_OFFSET) / GRID_SCALE,
         # convert absolute coordinate to number of beats since start
         round((coord[2] * bpm / BPM_DIVISOR) * 64 - startMeasure) / 64,
     ])
@@ -231,8 +231,8 @@ def export_clipboard(data: DataContainer, realign_start: bool = True):
             last = wall[0, 2]
         dest_list, wall_dict = wall_to_synth(data.bpm, wall)
         clipboard[dest_list].append(wall_dict)
-    
-    if not realign_start:
+
+    if realign_start:
         # position of selection start in beats*64 
         clipboard["startMeasure"] = round(first * 64)
         # position of selection start in ms
@@ -240,8 +240,8 @@ def export_clipboard(data: DataContainer, realign_start: bool = True):
         # length of the selection in milliseconds
         # and yes, the editor has a typo, so we need to missspell it too
         clipboard["lenght"] = last * MS_PER_MIN / data.bpm
-    else:
-        clipboard["lenght"] = (last - first) * MS_PER_MIN / data.bpm
+    # always update length
+    clipboard["lenght"] = (last - first) * MS_PER_MIN / data.bpm
 
     pyperclip.copy(json.dumps(clipboard))
 
